"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const node_assert_1 = require("node:assert");
const internal_1 = require("@fluidframework/test-runtime-utils/internal");
const runCounter_js_1 = require("../runCounter.js");
describe("RunCounter", () => {
    it("should start with zero runs", () => {
        const runCounter = new runCounter_js_1.RunCounter();
        node_assert_1.strict.strictEqual(runCounter.runs, 0);
        node_assert_1.strict.strictEqual(runCounter.running, false);
    });
    it("should increment runs when running", () => {
        const runCounter = new runCounter_js_1.RunCounter();
        runCounter.run(() => {
            node_assert_1.strict.strictEqual(runCounter.runs, 1);
            node_assert_1.strict.strictEqual(runCounter.running, true);
        });
        node_assert_1.strict.strictEqual(runCounter.runs, 0);
        node_assert_1.strict.strictEqual(runCounter.running, false);
    });
    it("should handle nested runs correctly", () => {
        const runCounter = new runCounter_js_1.RunCounter();
        runCounter.run(() => {
            node_assert_1.strict.strictEqual(runCounter.runs, 1);
            runCounter.run(() => {
                node_assert_1.strict.strictEqual(runCounter.runs, 2);
            });
            node_assert_1.strict.strictEqual(runCounter.runs, 1);
        });
        node_assert_1.strict.strictEqual(runCounter.runs, 0);
    });
    it("should return the result of the action", () => {
        const runCounter = new runCounter_js_1.RunCounter();
        const result = runCounter.run(() => 42);
        node_assert_1.strict.strictEqual(result, 42);
    });
    it("should decrement runs even if the action throws", () => {
        const runCounter = new runCounter_js_1.RunCounter();
        node_assert_1.strict.throws(() => {
            runCounter.run(() => {
                throw new Error("test error");
            });
        }, /test error/);
        node_assert_1.strict.strictEqual(runCounter.runs, 0);
        node_assert_1.strict.strictEqual(runCounter.running, false);
    });
});
describe("BatchRunCounter", () => {
    describe("BatchRunCounter", () => {
        it("should start with zero runs and undefined resubmitInfo", () => {
            const batchRunCounter = new runCounter_js_1.BatchRunCounter();
            node_assert_1.strict.strictEqual(batchRunCounter.runs, 0);
            node_assert_1.strict.strictEqual(batchRunCounter.running, false);
            node_assert_1.strict.strictEqual(batchRunCounter.resubmitInfo, undefined);
        });
        it("should set and clear resubmitInfo during run", () => {
            const batchRunCounter = new runCounter_js_1.BatchRunCounter();
            const info = { batchId: "foo", staged: true };
            batchRunCounter.run(() => {
                node_assert_1.strict.strictEqual(batchRunCounter.resubmitInfo, info);
            }, info);
            node_assert_1.strict.strictEqual(batchRunCounter.resubmitInfo, undefined);
        });
        it("should pass through return value from run", () => {
            const batchRunCounter = new runCounter_js_1.BatchRunCounter();
            const result = batchRunCounter.run(() => 123, undefined);
            node_assert_1.strict.strictEqual(result, 123);
        });
        it("should decrement runs and clear resubmitInfo even if action throws", () => {
            const batchRunCounter = new runCounter_js_1.BatchRunCounter();
            const info = { batchId: "foo", staged: true };
            node_assert_1.strict.throws(() => {
                batchRunCounter.run(() => {
                    throw new Error("fail");
                }, info);
            }, /fail/);
            node_assert_1.strict.strictEqual(batchRunCounter.runs, 0);
            node_assert_1.strict.strictEqual(batchRunCounter.resubmitInfo, undefined);
        });
        it("should not allow reentrancy if outer call sets resubmitInfo", () => {
            const batchRunCounter = new runCounter_js_1.BatchRunCounter();
            node_assert_1.strict.throws(() => {
                batchRunCounter.run(() => {
                    batchRunCounter.run(() => { });
                }, { batchId: "foo", staged: true });
            }, (e) => (0, internal_1.validateAssertionError)(e, "Reentrancy not allowed in BatchRunCounter"));
        });
    });
});
//# sourceMappingURL=runCounter.spec.js.map